; =================================================================
; Level
; =================================================================

; *******************************
; * Planeado *
; 
; VRAM
; $000 - Tiles del nivel
; $180 - Tiles del fondo del nivel
; $300 - Premios (los items)
; $400 - Enemigos
; $580 - Player
; 
; $780 - HUD
; *******************************

; =================================================================
; ------------------------------------------------
; Includes
; ------------------------------------------------

		include	"engine/modes/level/subs/level.asm"
 		include	"engine/modes/level/subs/object.asm"
 		
; =================================================================
; ------------------------------------------------
; RAM
; ------------------------------------------------

		rsset RAM_ModeBuffer
RAM_Player	rs.b $100			;$FFFF6080
RAM_LevelPrizes	rs.b max_lvlprz			;$FFFF6180
RAM_LevelObjPos	rs.l (max_lvlobj)*2		;$FFFFB180 | $FFCC CCCC XXXX YYYY | flags CCCCCC code | X pos | Y pos | 
RAM_LevelBuffer	rs.b (sizeof_level)		;$FFFFB580
RAM_ObjBuffer	rs.b (sizeof_obj*max_objects)	;$FFFFB5B4
RAM_LayEditor	rs.b $100

sizeof_all	rs.l 0
 
; =================================================================
; ------------------------------------------------
; RAM: Debugger
; ------------------------------------------------

		rsreset
DbgWdX		rs.w 1
DbgWdY		rs.w 1

; =================================================================
; ------------------------------------------------
; Init
; ------------------------------------------------

mode_Level:
		bsr	Mode_FadeOut
		
		move.w	#$2700,sr
		
; -----------------------------------
; Cleanup
; -----------------------------------

		bsr	Vdp_ClearPlanes
		bsr	Mode_Cleanup

; -----------------------------------
; Init stuff
; -----------------------------------

    		move.l	#VInt_Default,(RAM_VIntAddr)
    		move.l	#Hint_Default,(RAM_HIntAddr)
		lea	(RAM_VdpRegs),a0
     		bclr	#bit_vdpHint,(a0)
		move.b	#1,vdpReg_PlnSize(a0)
		move.b	#3,vdpReg_Scroll(a0)
		lea	(RAM_VdpRegs),a0
		move.b	#vdp_H40,vdpReg_HMode(a0)
          	clr.b	vdpReg_WindowY(a0)
 		bsr	Vdp_Update

; -----------------------------------

 		if MARS
		moveq	#0,d1
		moveq	#$E,d0
		bsr	Mars_DoTask_Master
      		move.w	#0,(marsreg+comm4)
   		moveq	#$10,d0
  		bsr	Mars_Task_Master
  		bsr	Mars_Wait_Master
 		bclr	#FM,(marsreg+access)
 		endif
 		
; -----------------------------------
; Init level 
; -----------------------------------

		bsr	Level_init
		bsr	Objects_init
		
; -----------------------------------
; Load
; -----------------------------------
		
  		lea	(Art_DebugFont),a0
   		move.w	#$580,d0
   		move.w	#((Art_DebugFont_End-Art_DebugFont)/4)-1,d1
   		bsr	VDP_SendData_L

		move.w	(RAM_SharedBuffer),d0
		bsr	Level_GrabData
  		bsr	Level_GrabBgData
		
 		bsr	Level_draw
 		bsr	Level_run
 		bsr	Objects_Run
 
; -----------------------------------
; Cool debugger
; -----------------------------------

 		bsr	Debug_Init
		
; -----------------------------------
; Object
; -----------------------------------

		;Player object
   		move.l	#Obj_player,(RAM_ObjBuffer+obj_code)
		move.l	#($0000<<16)|$0000,d0
  		bsr	Plyr_SetStartPos	
 		lea	(Pal_Player),a0
 		lea	(RAM_PalFadeBuff+$60),a1
 		move.w	#(16)-1,d0
   		bsr	LoadData_Word
  		
; -----------------------------------

		move.w	#$2000,sr
      		
		bsr	Mode_FadeIn
		
; =================================================================
; ------------------------------------------------
; Loop
; ------------------------------------------------

LevelMode_Loop:
		bsr	Sprites_Reset
		bsr	VSync
		
 		bsr	Objects_Run
 		bsr	Level_run
 		
  		bsr	Debug_Upd
 		
		btst	#bitJoyStart,(RAM_Joypads+OnPress)
		beq.s	LevelMode_Loop
		clr.b	(RAM_GameMode)
		rts
		
; =================================================================
; ------------------------------------------------
; Hblank
; ------------------------------------------------

Hint_Level:
 		btst	#1,(RAM_VIntWait)
 		beq	@Wait
 		bclr	#1,(RAM_VIntWait)
 		
  		move.l	#$84058220,($C00004)

  		move.l	#$40000010,($C00004).l
  		move.l	(RAM_VerBufferHint),($C00000).l
  		
  		lea	(RAM_PalBufferHint),a0
 		move.l	#$C0000000,($C00004).l
  		move.w	#$3F,d0
@PalBuf:
  		move.w	(a0)+,($C00000).l
  		dbf	d0,@PalBuf
 		
@WaitVDP:
  		move.w	($C00004).l,d0
  		andi.w	#4,d0
 		beq.s	@WaitVDP
 
 		lea	(RAM_SprBufferHint),a0
 		move.l	#$78000003,($C00004).l
 		move.w	#$7F,d0
@LoopSprH:
 		move.l	(a0)+,($C00000).l
 		dbf	d0,@LoopSprH
; 		
		move.w	#$8A6F,($C00004)
@Wait:
 		rts
		
; =================================================================
; ------------------------------------------------
; VBlank
; ------------------------------------------------

Vint_Level:
  		move.l	#$82308407,($C00004)

  		bsr	DMA_Read
	 	bsr	Pads_Read
 		
		lea	(RAM_PalBuffer),a0
 		move.l	#$C0000000,($C00004).l
 		move.w	#$3F,d0
@PalBuf:
		move.w	(a0)+,($C00000).l
 		dbf	d0,@PalBuf

		lea	(RAM_SprBuffer),a0
		move.l	#$78000003,($C00004).l
		move.w	#$9F,d0
@SprBuf:
		move.l	(a0)+,($C00000).l
		dbf	d0,@SprBuf
		
 		lea	(RAM_VerBuffer),a0
 		move.l	#$40000010,($C00004).l
 		move.w	#$F,d0
@VerBuf:
 		move.l	(a0)+,($C00000).l
 		dbf	d0,@VerBuf
 
 		lea	(RAM_HorBuffer),a0
 		move.l	#$7C000003,($C00004).l
 		move.w	#(224/2)-1,d0
@HorBuf:
 		move.l	(a0)+,($C00000).l
 		dbf	d0,@HorBuf
 
;  		btst	#2,(RAM_VIntWait)
;  		bne.s	@HintPrio
 		bsr	PalFade
  		bsr	SMEG_Upd
  		
@HintPrio:
 		bset	#1,(RAM_VIntWait)
		rts
		
; =================================================================
; ------------------------------------------------
; Subs
; ------------------------------------------------

; d0 - ID

Level_GrabData:
		moveq	#0,d1
		move.w	d0,d1
		lsl.w	#4,d1
		lea	LevelList(pc,d1.w),a2
		movea.l	(a2)+,a0
 		bsr	Level_load
 		
		movea.l	(a2)+,a0
 		move.w	#0,d0
 		move.w	(a0)+,d1
 		bsr	VDP_SendData_L
 		
		movea.l	(a2)+,a0
 		lea	(RAM_PalFadeBuff),a1
 		move.w	(a0)+,d0
  		bsr	LoadData_Word
  		
		move.l	(a2)+,d0
		tst.l 	d0
		beq.s 	@Finish
		movea.l	d0,a1
@NextEntry:
		tst.w	(a1)
		bmi.s	@Finish
		move.l	(a1)+,a0		;Addr
 		move.w	(a1)+,d0		;VRAM
 		move.w	(a1)+,d1		;Size
 		bsr	VDP_SendData_L
 		bra.s	@NextEntry
 		
@Finish:
		rts
	
; ------------------------------------------------

Level_GrabBgData:
 		lea	(Art_LvlBG_Test),a0
 		move.w	#$180,d0
 		move.w	(a0)+,d1
 		bsr	VDP_SendData_L
 		lea	(Pal_LevelBg_Test),a0
 		lea	(RAM_PalFadeBuff+$20),a1
		move.w	#((Pal_LevelBg_Test_End-Pal_LevelBg_Test)/2)-1,d0
  		bsr	LoadData_Word
     		lea	(Map_LvlBG_Test),a0
 		moveq	#1,d0
		move.l	#($0000<<16)|$0000,d1
 		move.l	#((((512)/8)-1)<<16)|(((256)/8)-1),d2
		move.w	#$2180,d3
		bra	VDP_LoadMaps
; 		rts
		
; =================================================================
; ------------------------------------------------
; Data
; ------------------------------------------------

LevelList:
 		dc.l test_level,Art_Level_Test,Pal_Level_Test,artdata_Level_Test
  		even
  		
asc_Debug:
		dc.b "Ly Bl Cl Pz Ob         00000000 00000000"
		even
		
		
; ------------------------------------------------
; Level editor
; ------------------------------------------------

Debug_Init:
		if SegaCD=0
		if MARS=0
		
 		lea	(RAM_LayEditor),a4
 		
 		moveq	#2,d0
 		moveq	#0,d1
 		move.w	#$6560,d2
 		lea	(asc_Debug),a0
 		bsr	VDP_LoadAsc
 		
 		move.w	#$9100,DbgWdX(a4)
 		move.w	#$9201,DbgWdY(a4)
 		move.w	DbgWdX(a4),($C00004).l
 		move.w	DbgWdY(a4),($C00004).l
 		
 		endif
 		endif
 		
 		
Debug_Upd:
		if SegaCD=0
		if MARS=0
		
		lea	(RAM_LayEditor),a4
		
 		moveq	#2,d0
 		move.l	#($0017<<16)|$0000,d1
 		move.w	#$6560+"0",d2
		move.w	(RAM_ObjBuffer+obj_x),d3
		bsr	VDP_ShowVal_W
 		moveq	#2,d0
 		move.l	#($001B<<16)|$0000,d1
 		move.w	#$6560+"0",d2
		move.w	(RAM_ObjBuffer+obj_y),d3
		bsr	VDP_ShowVal_W
		
  		moveq	#2,d0
  		move.l	#($0020<<16)|$0000,d1
  		move.w	#$6560+"0",d2
 		move.w	(RAM_LevelBuffer+lvl_x),d3
 		bsr	VDP_ShowVal_W
  		moveq	#2,d0
  		move.l	#($0024<<16)|$0000,d1
  		move.w	#$6560+"0",d2
 		move.w	(RAM_LevelBuffer+lvl_y),d3
 		bsr	VDP_ShowVal_W
;     	
;   		moveq	#2,d0
;   		move.l	#($0009<<16)|$0000,d1
;   		move.w	#$6560+"0",d2
;  		move.l	($FFFF0000),d3
;  		bsr	VDP_ShowVal_L
;   		moveq	#2,d0
;   		move.l	#($0012<<16)|$0000,d1
;   		move.w	#$6560+"0",d2
;  		move.l	($FFFF0004),d3
;  		bsr	VDP_ShowVal_L
;   		moveq	#2,d0
;   		move.l	#($001B<<16)|$0000,d1
;   		move.w	#$6560+"0",d2
;  		move.l	($FFFF0008),d3
;  		bsr	VDP_ShowVal_L		
 		
  		endif
 		endif
 		
 		rts
 		