; =================================================================
; Level system
; 
; Notes:
; FG Adress: $C000
; BG Adress: $E000
; =================================================================

; ------------------------------------------------
; Variables
; ------------------------------------------------

Plane_FG	equ	$40000003
Plane_WD	equ	$50000003
Plane_BG	equ	$60000003
vdp_Ypos_40	equ	$800000

max_lvlprz	equ	$5000
max_lvlobj	equ	64

; -----------------------

			rsreset
lvl_layout		rs.l	1
lvl_blocks		rs.l	1
lvl_collision		rs.l	1
lvl_objects		rs.l	1
lvl_x			rs.w	1
lvl_y			rs.w	1
lvl_x_size		rs.w	1
lvl_y_size		rs.w	1
lvl_timer		rs.w	1
lvl_drawdir		rs.b	1
lvl_prio		rs.b	1
lvl_status		rs.b	1
lvl_settings		rs.b	1
sizeof_level		rs.l	0
 
; -----------------------
; lvl_status

bit_onscroll		equ	0

; -----------------------

; lvl_prio
bit_hortype		equ	1
bit_split		equ	0

; -----------------------

; lvl_drawdir
bit_drw_r		equ	0
bit_drw_l		equ	1
bit_drw_d		equ	2
bit_drw_u		equ	3
bit_drw_r_2p		equ	4
bit_drw_l_2p		equ	5
bit_drw_d_2p		equ	6
bit_drw_u_2p		equ	7

; ------------------------------------------------
; RAM
; ------------------------------------------------
 
; RAM_LevelPrizes		rs.b max_lvlprz
; RAM_LevelObjPos		rs.l (max_lvlobj*2)			;$FFCC CCCC XXXX YYYY flags CCCCCC code | X pos | Y pos | 
; RAM_LevelBuffer		rs.b (sizeof_level)*2
			
; =================================================================
; ------------------------------------------------
; Subs
; ------------------------------------------------

; ------------------------------------------------
; Level_load
;
; Set a level
; 
; Input:
; a0 - level data
;	dc.l @floor
;	dc.l @interblocks
;	dc.l @collision
;	dc.l @objects
;	dc.w x_size
;	dc.w y_size
;	dc.b settings
;	dc.b 0
;	dc.b (layout_data)
;
; d0 | LONG - Xpos|Ypos
; d1 | BYTE - lvl_prio flags
; 
; Uses:
; d3-d7,a0-a1
; ------------------------------------------------

Level_load:
		lea	(RAM_LevelBuffer),a4
		bclr	#bit_hortype,lvl_prio(a4)
     		cmp.b	#vdp_H40,(RAM_VdpRegs+vdpReg_HMode)
     		beq.s	@NormalHor
		bset	#bit_hortype,lvl_prio(a4)
@NormalHor:
 		move.l	(a0)+,lvl_blocks(a4)
 		move.l	(a0)+,lvl_collision(a4)
 		move.l	(a0)+,d2			;RLEd prizes to RAM
 		move.l	(a0)+,lvl_objects(a4)
;  		adda	#1,a0
;  		move.b	(a0)+,lvl_settings(a4)
 		move.w	(a0)+,lvl_x_size(a4)
 		move.w	(a0)+,lvl_y_size(a4)
 		move.l	a0,lvl_layout(a4)
 		
 		and.b	#%00000110,(RAM_VdpRegs+vdpReg_HMode)
 		beq.s	@notdouble
 		bset	#bit_split,lvl_prio(a4)
@notdouble:

; ----------------------------------		
; Load object from the list
; ----------------------------------

		lea	(RAM_LevelObjPos),a3
		move.w	#max_lvlobj/8,d1
@clrlist:
		clr.l	(a3)+
		clr.l	(a3)+
		clr.w	(a3)+
		dbf	d1,@clrlist
		
		movea.l	lvl_objects(a4),a3
		lea	(RAM_LevelObjPos),a4
		move.w	#max_lvlobj/8,d1
@NextObj:
		tst.l	(a3)
		beq.s	@Nothing
 		move.l	(a3)+,d0
 		and.l 	#$FFFFFF,d0
 		move.l	d0,(a4)+
		move.l	(a3)+,(a4)+
		move.w	(a3)+,(a4)+
		dbf	d1,@NextObj
@Nothing:

; ----------------------------------

 		tst.l	d2
 		beq.s	@rleprz_end
 		bmi.s	@rleprz_end
 		movea.l	d2,a3
 		lea	(RAM_LevelPrizes),a4
 		moveq	#0,d1
@rle_prz:
 		moveq	#0,d0
		move.b	(a3)+,d0
 		cmp.b	#$FF,d0
 		beq.s	@rleprz_end
 		sub.w	#1,d0
 		move.b	(a3)+,d1
@rle_copy:
 		move.b	d1,(a4)+
 		dbf	d0,@rle_copy
 		bra.s	@rle_prz
@rleprz_end:

 		rts
		
; ------------------------------------------------
; Level_draw
;
; Draws the level fully
; The level must be loaded BEFORE calling this
; 
; Uses:
; d0-d7,a1-a6
; ------------------------------------------------
		
Level_draw:
 
; ----------------------------------		
; draw the level
; ----------------------------------

		lea	(RAM_LevelBuffer),a6
		move.l	#Plane_FG,d4		; VDP Address
  		move.w	lvl_x(a6),d2		; X pos
  		move.w	lvl_y(a6),d3		; Y pos
   		lsr.w	#4,d3			; Ypos: xxx0 > 0xxx
   		and.w	#$F,d3			; 0xxx > 00xx
   		lsl.w	#8,d3			; 00xx > xx00
   		and.w	#$3FFF,d3
   		lsr.w	#4,d2			; Xpos: xxx0 > 0xxx
    		lsl.w	#2,d2			; 0xxx * 2
    		swap	d4
   		add.w	d3,d4			; +Y VDP
  		add.b	d2,d4			; +X VDP
       		and.b	#$7F,d4
      		swap	d4

; ----------------------------------

		movea.l	lvl_layout(a6),a5
		lea	(RAM_LevelPrizes),a4
 		move.w	lvl_y(a6),d2
 		lsr.w	#4,d2
 		mulu.w	lvl_x_size(a6),d2
;  		lsl.w	#1,d2
 		adda	d2,a5		
  		move.w	lvl_x(a6),d2
  		lsr.w	#4,d2
;    		lsl.w	#1,d2
   		adda 	d2,a5
		
; ----------------------------

     		move.w	#$2700,sr
 		move.w	lvl_y_size(a6),d1
  		cmp.w	#(32/2),d1
  		blt.s	@low_y
  		move.w	#(32/2),d1
@low_y:
 		sub.w	#1,d1
		move.w	d1,d5
  		move.w	lvl_x_size(a6),d2
  		cmp.w	#(64/2),d2
  		blt.s	@low_x
  		move.w	#(64/2),d2
@low_x:
 		sub.w	#1,d2
 		
 		movea.l	a5,a3
 		movea.l	a4,a2
@next_x:
 		move.l	d4,d3
 		move.w	d5,d1
@next_y:
 		moveq	#0,d0
 		move.b	(a5),d0
		movea.l	lvl_blocks(a6),a1
  		lsl.w	#3,d0
  		adda 	d0,a1
  		move.l	(a1)+,d0
  		move.l	(a1)+,d6
  		
  		tst.b	(a4)
  		beq.s	@NotPrize
  		
  		moveq	#0,d0
   		move.b	(a4),d0
  		lea	vram_prizes(pc),a1
    		lsl.w	#3,d0
    		adda 	d0,a1
    		move.l	(a1)+,d0
    		move.l	(a1)+,d6
  		
@NotPrize:
 		swap	d0
 		swap	d6
      		btst	#bit_split,lvl_prio(a6)
      		beq.s	@DontDiv
      		
      		move.w	d0,d7
      		and.w	#$F800,d7
 		lsr.w	#1,d0
 		and.w	#$7FF,d0
 		or.w	d7,d0
      		move.w	d6,d7
      		and.w	#$F800,d7
 		lsr.w	#1,d6
 		and.w	#$7FF,d6
 		or.w	d7,d6
 		
@DontDiv:
 		move.l	d3,($C00004)
   		move.w	d0,($C00000)
   		move.w	d6,($C00000)
  		swap	d3
  		add.w	#$80,d3
		and.w	#$4FFF,d3
    		swap	d3

      		btst	#bit_split,lvl_prio(a6)
      		bne.s	@Split16
 		swap	d0
 		swap	d6
 		move.l	d3,($C00004)
   		move.w	d0,($C00000)
   		move.w	d6,($C00000)
  		swap	d3
  		add.w	#$80,d3
		and.w	#$4FFF,d3
    		swap	d3
@Split16:

  		move.w	lvl_x_size(a6),d0
  		adda	d0,a4
;   		lsl.w	#1,d0
  		adda	d0,a5
    		dbf	d1,@next_y
    		
   		swap	d4
   		add.b	#4,d4
    		and.b	#$7F,d4
    		swap	d4
    		
    		adda	#1,a3
    		adda	#1,a2
 		movea.l	a3,a5
 		movea.l	a2,a4
     		dbf	d2,@next_x
     		
; ----------------------------------
; Yes, a copy-paste
; ----------------------------------

		lea	(RAM_LevelObjPos),a5
		lea	(RAM_ObjBuffer+(sizeof_obj*8)),a4
		moveq	#0,d3
@next_obj:
		tst.l	(a5)
		beq	@finish
 		btst	#7,(a5)			;ON SCREEN flag?
 		bne	@next
  		
@found_obj:
  		;$FFB520
  		moveq	#(max_objects-16)-1,d4
@nxtav_obj:
		tst.l	obj_code(a4)	
		beq.s	@valid
		adda	#sizeof_obj,a4
		dbf	d4,@nxtav_obj
		
@valid:
		move.w	4(a5),obj_x(a4)
		move.w	6(a5),obj_y(a4)
		move.b	d3,obj_spwnindx(a4)
		
   		moveq	#0,d1
    		move.b	8(a5),d1
    		bclr	#bitobj_dir,obj_status(a4)
    		btst	#0,d1
    		beq.s	@dont_flip_l
  		bset	#bitobj_dir,obj_status(a4)
@dont_flip_l:
 		move.b	(a5),d1
 		and.w	#$7F,d1
		move.b	d1,obj_subid(a4)
 		move.l	(a5),d0
  		and.l 	#$FFFFFF,d0
 		move.l	d0,obj_code(a4)
 		bset	#7,(a5)			;SET ON SCREEN flag
		adda	#sizeof_obj,a4
@next:
		add.w	#1,d3
		adda	#$A,a5
		bra	@next_obj
@finish:

; 		bclr	#bit_objrefresh,lvl_status(a6)
     		move.w	#$2000,sr
     		rts
	
; =================================================================
; ------------------------------------------------
; Init
; ------------------------------------------------

Level_init:
		lea	(RAM_LevelBuffer),a6
		move.w	#sizeof_level-1,d0
@cleanup:
		clr.b	(a6)+
		dbf	d0,@cleanup
		
 		lea	(Art_LvlPrizes),a0
 		move.w	#$300,d0
 		move.w	(a0)+,d1
 		bsr	VDP_SendData_L
		rts
		
; =================================================================
; ------------------------------------------------
; Run
; ------------------------------------------------

Level_run:
		lea	(RAM_LevelBuffer),a6
		bsr	Level_DrawOnMove
  		bsr	Level_refreshobj
; 		rts
	
; ------------------------------------------------
; ----------------------------------		
; Level scroll/deform
; ----------------------------------

Level_Scroll:
		bra.s	@new_one

 		lea	(RAM_HorBuffer),a5
		move.w	#(56)-1,d3
		move.w	lvl_x(a6),d1
		move.w	d1,d0
		neg.w	d0
		swap	d0
		lsr.w	#5,d1
		move.w	d1,d0
		neg.w	d0
		move.w	#(224)-1,d3
@doline:
 		move.l	d0,(a5)+
		dbf	d3,@doline
		
		move.w	lvl_y(a6),d1
		move.w	d1,d0
		swap	d0
		lsr.w	#5,d1
		move.w	d1,d0
		lea	(RAM_VerBuffer),a5
		move.l	d0,(a5)
 		rts
 		
 		
@new_one:
 		lea	(RAM_HorBuffer),a5
		move.w	lvl_x(a6),d1
		move.w	d1,d2
		move.w	d1,d0
		neg.w	d0
		swap	d0
		
		move.w	d2,d1
		lsr.w	#6,d1
		neg.w	d1
		move.w	d1,d0
		move.w	#(56)-1,d3
@doline23:
 		move.l	d0,(a5)+
		dbf	d3,@doline23
		move.w	d2,d1
		lsr.w	#5,d1
		neg.w	d1
		move.w	d1,d0
		move.w	#(72)-1,d3
@doline_2:
 		move.l	d0,(a5)+
		dbf	d3,@doline_2
		
		moveq	#0,d4
		move.w	d2,d1
		lsr.w	#2,d1
		neg.w	d1
		move.w	d1,d0
 		move.w	lvl_x(a6),d1
 		and.l	#$FFFF,d1
 		lsl.l	#8,d1
		move.w	#(96)-1,d3
@doline_3:
		swap	d4
		move.w	d4,d0
		lsr.w	#1,d0
		swap	d4
 		move.l	d0,(a5)+
 		sub.l	d1,d4
		dbf	d3,@doline_3
		
		;Vertical
		move.w	lvl_y(a6),d2
		move.w	d2,d0
		lsr.w	#4,d0
		lea	(RAM_VerBuffer),a5
		move.w	d2,(a5)+
 		move.w	d0,(a5)+
 		
 		add.w	#1,lvl_timer(a6)
		rts
		
; ----------------------------------		
; load level objects while moving
; ----------------------------------

Level_RefreshObj:
		lea	(RAM_LevelObjPos),a5
		lea	(RAM_ObjBuffer+(sizeof_obj*8)),a4
		moveq	#0,d3
@next_obj:
		tst.l	(a5)
		beq	@finish
 		btst	#7,(a5)			;ON SCREEN flag?
 		bne	@next
		
   		move.w	lvl_x(a6),d0
       		move.w	4(a5),d1
    		move.w	d0,d2
   		add.w	#320+$40,d2
   		sub.w	#$40,d0
      		cmp.w	d0,d1
       		blt	@found_obj
     		cmp.w	d2,d1
      		bge	@found_obj
  		
    		move.w	lvl_y(a6),d0
       		move.w	6(a5),d1
     		move.w	d0,d2
    		add.w	#224+$40,d2
    		sub.w	#$40,d0
      		cmp.w	d0,d1
       		blt	@found_obj
      		cmp.w	d2,d1
       		bge	@found_obj

   		bra.s	@next
  		
@found_obj:
  		;$FFB520
  		moveq	#(max_objects-16)-1,d4
@nxtav_obj:
		tst.l	obj_code(a4)
		beq.s	@valid
		adda	#sizeof_obj,a4
		dbf	d4,@nxtav_obj
		bra	@next
		
@valid:
		move.w	4(a5),obj_x(a4)
		move.w	6(a5),obj_y(a4)
		move.b	d3,obj_spwnindx(a4)
		
   		moveq	#0,d1
    		move.b	8(a5),d1
    		bclr	#bitobj_dir,obj_status(a4)
    		btst	#0,d1
    		beq.s	@dont_flip_l
  		bset	#bitobj_dir,obj_status(a4)
@dont_flip_l:
 		move.b	(a5),d1
 		and.w	#$7F,d1
		move.b	d1,obj_subid(a4)
 		move.l	(a5),d0
  		and.l 	#$FFFFFF,d0
 		move.l	d0,obj_code(a4)
 		bset	#7,(a5)			;SET ON SCREEN flag
		adda	#sizeof_obj,a4
@next:
		add.w	#1,d3
		adda	#$A,a5
		bra	@next_obj
@finish:
		rts
 		
; ----------------------------------	
		
; ----------------------------------		
; draw level in parts
; ----------------------------------

Level_DrawOnMove:

; ----------------------------------		
; draw right
; ----------------------------------

		btst	#bit_drw_r,lvl_drawdir(a6)
		beq	@dont_draw_r

		movea.l	lvl_layout(a6),a5
		lea	(RAM_LevelPrizes),a4
		adda	#$16,a5
		adda	#$16,a4
		move.w	lvl_y(a6),d2
		lsr.w	#4,d2
		tst.w	d2
		beq.s	@zero_y_r
		sub.w	#1,d2
@next_y_r:
 		move.w	lvl_x_size(a6),d0
 		adda 	d0,a4
;  		lsl.w	#1,d0
   		adda	d0,a5
   		dbf	d2,@next_y_r
@zero_y_r:
		move.w	lvl_x(a6),d2
 		lsr.w	#4,d2
		and.w	#$FFF,d2
 		adda 	d2,a4
;  		lsl.w	#1,d2
   		adda 	d2,a5
   		
		move.l	#Plane_FG,d1		; VDP Address
   		swap	d1
   		add.b	#$16*4,d1
   		swap	d1
		move.w	lvl_x(a6),d2		; X pos
		move.w	lvl_y(a6),d3		; Y pos
		lsr.w	#4,d3			; Ypos: xxx0 > 0xxx
		and.w	#$F,d3			; 0xxx > 00xx
		lsl.w	#8,d3			; 00xx > xx00
 		lsr.w	#4,d2			; Xpos: xxx0 > 0xxx
  		lsl.w	#2,d2			; 0xxx * 2
  		swap	d1
		add.w	d3,d1			; +Y VDP
		add.b	d2,d1			; +X VDP
       		and.b	#$7F,d1
   		swap	d1
		bsr	@draw_line_y
		
@already_x_r:
		bclr	#bit_drw_r,lvl_drawdir(a6)
@dont_draw_r:

; ----------------------------------		
; draw left
; ----------------------------------

		btst	#bit_drw_l,lvl_drawdir(a6)
		beq	@dont_draw_l

		movea.l	lvl_layout(a6),a5
		lea	(RAM_LevelPrizes),a4
;  		adda	#$14*2,a5
;  		adda	#$14,a4
		move.w	lvl_y(a6),d2
		lsr.w	#4,d2
		tst.w	d2
		beq.s	@zero_y_l
		sub.w	#1,d2
@next_y_l:
 		move.w	lvl_x_size(a6),d0
 		adda 	d0,a4
;  		lsl.w	#1,d0
   		adda	d0,a5
   		dbf	d2,@next_y_l
@zero_y_l:
		move.w	lvl_x(a6),d2
 		lsr.w	#4,d2
 		adda 	d2,a4
 		and.w	#$FFF,d2
;  		lsl.w	#1,d2
   		adda 	d2,a5
   		
		move.l	#Plane_FG,d1		; VDP Address
;    		swap	d1
;     		add.b	#$14*4,d1
;     		swap	d1
		move.w	lvl_x(a6),d2		; X pos
		move.w	lvl_y(a6),d3		; Y pos
		lsr.w	#4,d3			; Ypos: xxx0 > 0xxx
		and.w	#$F,d3			; 0xxx > 00xx
		lsl.w	#8,d3			; 00xx > xx00
 		lsr.w	#4,d2			; Xpos: xxx0 > 0xxx
  		lsl.w	#2,d2			; 0xxx * 2
  		swap	d1
		add.w	d3,d1			; +Y VDP
		add.b	d2,d1			; +X VDP
       		and.b	#$7F,d1
   		swap	d1
		bsr	@draw_line_y
		
		bclr	#bit_drw_l,lvl_drawdir(a6)
@dont_draw_l:

; ----------------------------------		
; draw down
; ----------------------------------

		btst	#bit_drw_d,lvl_drawdir(a6)
		beq	@dont_draw_d
		bclr	#bit_drw_d,lvl_drawdir(a6)
		
 		movea.l	lvl_layout(a6),a5
 		lea	(RAM_LevelPrizes),a4
 		move.w	lvl_y(a6),d2
  		lsr.w	#4,d2
 		add.w	#$E,d2
 		mulu.w	lvl_x_size(a6),d2
 		adda	d2,a4
;  		lsl.w	#1,d2
 		adda	d2,a5
 		move.w	lvl_x(a6),d2
  		lsr.w	#4,d2
  		adda 	d2,a4
  		and.w	#$FFF,d2
;   		lsl.w	#1,d2
    		adda 	d2,a5
    		
		move.l	#Plane_FG+(vdp_Ypos_40*($E*2)),d1
 		move.w	lvl_y(a6),d2
 		lsr.w	#4,d2
 		and.w	#$F,d2
 		lsl.w	#8,d2
  		swap	d1
 		add.w	d2,d1
   		ror.w	#8,d1
  		bclr	#4,d1
  		ror.w	#8,d1
    		swap	d1
 		move.w	lvl_x(a6),d2	
  		lsr.w	#4,d2
   		lsl.w	#2,d2
   		swap	d1
 		add.b	d2,d1
    		and.b	#$7F,d1
    		swap	d1
    		
  		bsr	@draw_line_x
  		
@dont_draw_d:

; ----------------------------------		
; draw up
; ----------------------------------

		btst	#bit_drw_u,lvl_drawdir(a6)
		beq	@dont_draw_u
		bclr	#bit_drw_u,lvl_drawdir(a6)
		
 		movea.l	lvl_layout(a6),a5
 		lea	(RAM_LevelPrizes),a4
 		move.w	lvl_y(a6),d2
  		lsr.w	#4,d2
 		mulu.w	lvl_x_size(a6),d2
 		adda	d2,a4
;  		lsl.w	#1,d2
 		adda	d2,a5
 		move.w	lvl_x(a6),d2
  		lsr.w	#4,d2
  		adda 	d2,a4
  		and.w	#$FFF,d2
;   		lsl.w	#1,d2
    		adda 	d2,a5
    		
		move.l	#Plane_FG,d1
 		move.w	lvl_y(a6),d2
 		lsr.w	#4,d2
 		and.w	#$F,d2
 		lsl.w	#8,d2
  		swap	d1
 		add.w	d2,d1
   		ror.w	#8,d1
  		bclr	#4,d1
  		ror.w	#8,d1
    		swap	d1
 		move.w	lvl_x(a6),d2	
  		lsr.w	#4,d2
   		lsl.w	#2,d2
   		swap	d1
 		add.b	d2,d1
    		and.b	#$7F,d1
    		swap	d1
    		
  		bsr	@draw_line_x
  		
@dont_draw_u:

; ----------------------------------


		rts
	
; ----------------------------------		
; Draw level line vertically
; 
; Input:
; a6 - level buffer
; a5 - level layout data
; a4 - level prizes
; d1 - vdp address
;
; Breaks:
; a2-a3
; ----------------------------------

@draw_line_y:
		moveq	#$F,d6
		moveq	#0,d5
@next_block_y:
		move.b	(a5),d0
		movea.l	lvl_blocks(a6),a3
  		lsl.w	#3,d0
  		adda 	d0,a3
  		
   		move.l	(a3)+,d0
   		move.l	(a3)+,d2
;    		tst.l	lvl_prizes(a6)
;    		beq.s	@noprz_blk_1
;    		bmi.s	@noprz_blk_1
    		tst.b	(a4)
    		beq.s	@noprz_blk_1
   		
   		move.b	(a4),d0
		and.w	#$FF,d0
  		lea	vram_prizes(pc),a3
    		lsl.w	#3,d0
    		adda 	d0,a3
    		move.l	(a3)+,d0
    		move.l	(a3)+,d2

@noprz_blk_1:
 		swap	d0
 		swap	d2
      		btst	#bit_split,lvl_prio(a6)
      		beq.s	@DontDiv
      		move.w	d0,d3
      		and.w	#$F800,d3
 		lsr.w	#1,d0
 		and.w	#$7FF,d0
 		or.w	d3,d0
      		move.w	d2,d3
      		and.w	#$F800,d3
 		lsr.w	#1,d2
 		and.w	#$7FF,d2
 		or.w	d3,d2
 		
@DontDiv:
 		move.l	d1,($C00004)
   		move.w	d0,($C00000)
   		move.w	d2,($C00000)
   		
@contprz_1:
  		swap	d1
  		add.w	#$80,d1
		and.w	#$4FFF,d1
    		swap	d1
  		
      		btst	#bit_split,lvl_prio(a6)
      		bne.s	@Split16
 		swap	d0
 		swap	d2
 		move.l	d1,($C00004)
   		move.w	d0,($C00000)
   		move.w	d2,($C00000)
@contprz_2:
  		swap	d1
  		add.w	#$80,d1
		and.w	#$4FFF,d1
    		swap	d1
@Split16:

 		move.w	lvl_x_size(a6),d0
 		adda	d0,a4
;  		lsl.w	#1,d0
   		adda	d0,a5
   		
   		move.l	d4,d3
   		and.l	#$FFF000FF,d4
   		lsr.l	#8,d3
   		add.w	#1,d3
   		and.l	#$FFF,d3
   		lsl.l	#8,d3
   		or.l	d3,d4
   		
   		dbf	d6,@next_block_y
   		rts

; ----------------------------------		
; Draw level line horizontally
; 
; Input:
; a6 - level buffer
; a5 - level layout data
;
; d1 - vdp address
;
; Breaks:
; a3
; ----------------------------------

@draw_line_x:
 		movea.l	a5,a1
 		movea.l	a4,a0
 		move.l	d1,d7
 		bsr	@do_it

 		moveq	#0,d0
 		move.w	lvl_x_size(a6),d0
  		adda	d0,a0
;   		lsl.w	#1,d0
  		adda	d0,a1
 
   		swap	d7
   		add.w	#$100,d7
  		and.w	#$4FFF,d7
     		swap	d7
     		move.l	d7,d1
  		movea.l	a1,a5
  		movea.l	a0,a4
 		
@do_it:
		move.w	#$1F,d6
		move.l	d1,d3

@next_block_x:
		moveq	#0,d5
		move.b	(a5),d0
		movea.l	lvl_blocks(a6),a3
  		lsl.w	#3,d0
  		adda 	d0,a3
    		
   		move.l	(a3)+,d0
   		move.l	(a3)+,d2
    		tst.b	(a4)
    		beq.s	@noprz_blk_2
   		move.b	(a4),d0
		and.w	#$FF,d0
  		lea	vram_prizes(pc),a3
    		lsl.w	#3,d0
    		adda 	d0,a3
    		move.l	(a3)+,d0
    		move.l	(a3)+,d2
    		
@noprz_blk_2:
 		swap	d0
 		swap	d2
      		btst	#bit_split,lvl_prio(a6)
      		beq.s	@DontDivY
      		move.w	d0,d5
      		and.w	#$F800,d5
 		lsr.w	#1,d0
 		and.w	#$7FF,d0
 		or.w	d5,d0
      		move.w	d2,d5
      		and.w	#$F800,d5
 		lsr.w	#1,d2
 		and.w	#$7FF,d2
 		or.w	d5,d2
@DontDivY:
 		move.l	d1,($C00004)
   		move.w	d0,($C00000)
   		move.w	d2,($C00000)
  		swap	d1
  		add.w	#$80,d1
		and.w	#$4FFF,d1
    		swap	d1

      		btst	#bit_split,lvl_prio(a6)
      		bne.s	@Split16Y
 		swap	d0
 		swap	d2
 		move.l	d1,($C00004)
   		move.w	d0,($C00000)
   		move.w	d2,($C00000)
  		swap	d1
  		add.w	#$80,d1
		and.w	#$4FFF,d1
    		swap	d1
@Split16Y:

   		swap	d3
   		add.b	#4,d3
   		and.b	#$7F,d3
   		swap	d3
   		move.l	d3,d1
   		
   		move.l	d4,d2
   		and.l	#$000FFFFF,d4
   		swap	d2
   		lsr.w	#4,d2
   		add.w	#1,d2
   		and.l	#$FFF,d2
   		lsl.w	#4,d2
   		swap	d2
   		or.l	d2,d4
   		
 		adda	#1,a4
   		adda	#1,a5
   		dbf	d6,@next_block_x
   		rts
   		
; ----------------------------------		
; Check if Prize is marked on RAM
; 
; Call this routine AFTER any of
; the Object_FindCol_(Where)
; subs
; 
; Input:
; d0 | BYTE - Prize XYID
;
; Output:
; beq - Used
; bne - Not Used
; 
; Uses:
; a4|d5
; ----------------------------------

level_CheckPrize:
		moveq	#0,d5
		tst.w	d5
; 		lea	(RAM_LevelPrizes),a4
; 		move.w	#max_lvlprz/4,d5
; @next:
; 		cmp.l	#-1,(a4)
; 		beq.s	@Quit
; 		cmp.l	(a4),d0
; 		beq.s	@found_it
; 		adda	#4,a4
; 		dbf	d5,@next
; @Quit:
; 		moveq	#-1,d5
; 		tst.w	d5
; 		rts
; @found_it:
; 		moveq	#0,d5
; 		tst.w	d5
; @already:
		rts
		
; ----------------------------------		
; Mark prize to RAM
; 
; Call this routine AFTER any of
; the Object_FindCol_(Where)
; subs
; 
; Input:
; d0 | BYTE - Prize XYID
;
; Output:
; a4 - Last prize location
;
; Uses:
; a4|d5
; ----------------------------------

level_MarkPrize:
; 		lea	(RAM_LevelPrizes),a4
; 		move.w	#max_lvlprz/4,d5
; @next:
; 		cmp.l	#-1,(a4)
; 		bne.s	@not_end
; 		clr.l	(a4)
; @not_end:
; 		tst.l	(a4)
; 		beq.s	@mark_it
; 		cmp.l	(a4),d0
; 		beq.s	@already
; 		adda	#4,a4
; 		dbf	d5,@next
; 		rts
; @mark_it:
; 		move.l	d0,(a4)
;  		move.l	#-1,4(a4)	;Mark -1 End-Of-List (save CPU)
; @already:
		rts
		
; =================================================================
; ------------------------------------------------
; Data
; ------------------------------------------------

vram_prizes:
		dc.w 0,0,0,0
		dc.w $300,$301,$302,$303
		dc.w $304|$6000,$305|$6000,$306|$6000,$307|$6000
		dc.w $308,$309,$30A,$30B
		dc.w $30C|$6000,$30D|$6000,$30E|$6000,$30F|$6000
		dc.w $310,$311,$312,$313
		dc.w $314,$315,$316,$317
		dc.w $318,$319,$31A,$31B
		dc.w $31C,$31D,$31E,$31F
		even
		
; ------------------------------------------------

col_SlopeData:
		dc.b $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		dc.b $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		dc.b $0F,$0E,$0D,$0C,$0B,$0A,$09,$08,$07,$06,$05,$04,$03,$02,$01,$00	;$02
		dc.b $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
		dc.b $0F,$0F,$0E,$0E,$0D,$0D,$0C,$0C,$0B,$0B,$0A,$0A,$09,$09,$08,$08
		dc.b $07,$07,$06,$06,$05,$05,$04,$04,$03,$03,$02,$02,$01,$01,$00,$00
		dc.b $08,$08,$09,$09,$0A,$0A,$0B,$0B,$0C,$0C,$0D,$0D,$0E,$0E,$0F,$0F
		dc.b $00,$00,$01,$01,$02,$02,$03,$03,$04,$04,$05,$05,$06,$06,$07,$07
		
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		even
		
		