; =================================================================
; Object system
; =================================================================

; =================================================================
; ------------------------------------------------
; RAM
; ------------------------------------------------

; RAM_ObjBuffer	rs.b (sizeof_obj*max_objects)

; =================================================================
; ------------------------------------------------
; Settings
; ------------------------------------------------

max_objects	equ	64

; =================================================================
; ------------------------------------------------
; Variables
; ------------------------------------------------

		rsreset
obj_code	rs.l	1			; Object code 
obj_size	rs.l	1			; Object size (see below)

obj_x		rs.l	1			; Object X Position
obj_y		rs.l	1			; Object Y Position
obj_x_spd	rs.l	1			; Object X Speed
obj_y_spd	rs.l	1			; Object Y Gravity

obj_anim_next	rs.w	1			; Object animation increment (obj_anim + obj_anim_next)
obj_anim_id	rs.w	1			; Object animation to read (current+saved)
obj_anim_spd	rs.b	1			; Object animation delay
obj_index	rs.b	1			; Object code index
obj_status	rs.b	1			; Object status
obj_subid	rs.b	1			; Object SubID
obj_frame	rs.b	1			; Object display frame
obj_spwnindx	rs.b	1			; Object respawn index
obj_col		rs.b	1			; Object collision
obj_null	rs.b	1			; (Align filler)

obj_ram		rs.b	$C			; Object RAM
sizeof_obj	rs.l	0
;      		inform 0,"(OBJECT SYSTEM) Object size: %h",sizeof_obj
		
; --------------------------------
; obj_size
; --------------------------------

		rsreset
objsize_l	rs.b	1
objsize_r	rs.b	1
objsize_u	rs.b	1
objsize_d	rs.b	1

; --------------------------------
; obj_status
; --------------------------------

bitobj_dir	equ	0		; set for Left
bitobj_air	equ	1		; set for Jumping
bitobj_solid	equ	2		; set to make it solid

; --------------------------------
; obj_col
; --------------------------------

bitobj_col	equ	0		; Set for Touching floor
bitobj_colu	equ	1		; 
bitobj_colr	equ	2		; set for Touching floor/wall/Ceiling
bitobj_coll	equ	3		;

; =================================================================
; ------------------------------------------------
; Init
; 
; Uses:
; a0,d0
; ------------------------------------------------

Objects_init:
 		lea	(RAM_ObjBuffer),a0
		move.w	#(sizeof_obj*max_objects)-1,d0
@ClrObjs:
 		clr.b	(a0)+
 		dbf	d0,@ClrObjs 
		rts
		
; =================================================================
; ------------------------------------------------
; Run Objects 
;
; Uses:
; ALL
; ------------------------------------------------

Objects_run:
 		lea	(RAM_ObjBuffer),a6
 		move.w	#(max_objects)-1,d7
@Next:
 		tst.l	(a6)
 		beq.s	@NoAddr
 		movem.l	a6/d7,-(sp)
  		movea.l	(a6),a5
  		jsr	(a5)
 		movem.l	(sp)+,a6/d7
@NoAddr:
 		adda	#sizeof_obj,a6
 		dbf	d7,@Next
		rts
		
; =================================================================
; ------------------------------------------------
; Subs
; ------------------------------------------------
	
; ----------------------------------------------
; Object_BuildSpr
; 
; Input:
; d0 - VRAM|Frame
; d1 - Mappings data address
; d2 - X-pos|Y-pos 
; d3 - Flags
;
; Output:
; d3 - New sprite link
; 
; Uses:
; a3-a5/d4
; ----------------------------------------------
 
Object_BuildSpr:
 		lea	(RAM_SprControl),a5
		movea.l	sprite_free(a5),a4
		
 		movea.l	d1,a3
 		lsl.w	#1,d0
 		adda	(a3,d0.w),a3
 		
		and.l	#$FFFF,d3
 		moveq	#0,d6
 		move.b	(a3)+,d6
 		beq	@EndAll
 		sub.w	#1,d6
@Next:

; ------------
; Ypos check
; ------------

; TODO: hacer el mismo fix de abajo despues

		move.w	d2,d1
 		clr.w	d0
 		move.b	(a3),d0
 		ext.w	d0
 		add.w	d0,d1
 		move.b	(RAM_VdpRegs+$C),d0
 		and.w	#%00000110,d0
 		beq.s	@normal
		add.w	#$70,d1	
@normal:
 		move.w	d1,(a4)+
		
; ------------
; Size
; ------------

 		move.b	1(a3),(a4)+		; Size
 		move.b	5(a5),(a4)+		; Link
 		add.w	#1,sprite_link(a5)
 
; ------------
; Vram
; ------------

  		clr.w	d0			; Vram
  		move.b	2(a3),d0
  		lsl.w	#8,d0
  		move.b	3(a3),d0
  		swap	d0
  		move.w	d0,d1
  		swap	d0
  		add.w	d0,d1
  	
   		btst	#0,d3			;Left flag?
   		beq.s	@Right
   		or.w	#$800,d1
@Right:
;  		move.b	(RAM_VdpRegs+$C),d0
;  		and.w	#%00000110,d0
;  		beq.s	@normalv
; 		lsr.w	#2,d1
; @normalv:
  		move.w	d1,(a4)+
		
; ------------
; Xpos check
; ------------

 		clr.w	d0
 		moveq	#0,d1
 		moveq	#0,d4
 		swap	d2
 		move.w	d2,d1
 		swap	d2
 		
   		btst	#0,d3			;Left flag?
  		bne	@LeftSprX
 		move.b	4(a3),d0
 		ext.w	d0
 		add.w	d0,d1
 		bra.s	@ContX

@LeftSprX:
 		move.b	4(a3),d0
 		ext.w	d0
 		sub.w	d0,d1
 		move.b	1(a3),d4
  		lsr.w	#2,d4
  		lsl.w	#3,d4
  		sub.w	d4,d1
  		sub.w	#8,d1			;TODO: mala idea

@ContX:
 		move.w	d1,(a4)+
 		adda 	#5,a3
 		dbf	d6,@Next
 		move.l	a4,sprite_free(a5)
@EndAll:
 		rts

; ----------------------------------------------
; Object_DynArt
; 
; Input:
; d0 | LONG - VRAM|Frame
; d1 | LONG - DPLC data
; d2 | LONG - Art data
;
; Uses:
; a3/d3-d7
; ----------------------------------------------
 
Object_DPLC:
		and.w	#$FF,d0
		movea.l	d1,a3
		lsl.w	#1,d0
    		adda	(a3,d0.w),a3
   		moveq	#0,d6
   		move.w	(a3)+,d6
   		beq	@EndAll
   		sub.w	#1,d6
   		swap	d0
   		and.w	#$7FF,d0
   		moveq	#0,d5
   		move.w	d0,d5		;d5 - VRAM (base)
   		move.l	d2,d7
@Next:
     		move.w	(a3),d1
     		lsr.w	#8,d1
     		move.w	d1,d4
     		lsr.w	#4,d4		;d4 - Next VRAM
     		and.w	#$F,d4
     		add.w	#1,d4
     		add.w	#$10,d1		;Size + 1
     		swap	d1		;Size|ROM+Here
      		move.w	(a3)+,d1
      		and.w	#$FFF,d1
     		lsl.w	#5,d1
     		move.l	d7,d0
      		moveq	#0,d3
      		move.w	d1,d3
      		add.l	d3,d0		;ROM Addr + Cell number
     		
		swap	d1		;(Broken)|Size
		move.w	d5,d2
   		bsr	DMA_Set
		add.w	d4,d5		;Next VRAM
   		dbf	d6,@Next
 
@EndAll:
 		rts
 
; OLD
;     		move.w	(a3)+,d1
;     		move.w	d1,d4
;     		lsr.w	#4,d4		;d4 - Next VRAM
;     		add.w	#1,d4
;     		add.w	#$10,d1		;Size + 1
;     		swap	d1		;Size|ROM+Here
;      		move.w	(a3)+,d1
;     		lsl.w	#5,d1
;     		move.l	d7,d0
;      		moveq	#0,d3
;      		move.w	d1,d3
;      		add.l	d3,d0		;ROM Addr + Cell number

; ----------------------------------------------
; Object_Show
; 
; Input:
; d0 | LONG - VRAM|Frame
; d1 | LONG - Maps data
; ----------------------------------------------

Object_Show:
		and.w	#$FF,d0
		lea	(RAM_LevelBuffer),a5
 		move.w	obj_x(a6),d2
 		sub.w	lvl_x(a5),d2
 		add.w	#$80,d2
 		cmp.w	#$40,d2
 		blt.s	@Return
		cmp.w	#(320+$80)+$40,d2
		bgt.s	@Return
 		swap	d2
 		
 		move.w	obj_y(a6),d2
		sub.w	lvl_y(a5),d2
 		add.w	#$80,d2
 		cmp.w	#$40,d2
 		blt.s	@Return
		cmp.w	#(224+$80)+$40,d2
		bgt.s	@Return

		bclr	#0,d3
		btst	#bitobj_dir,obj_status(a6)
		beq.s	@Right_LR
		bset	#0,d3
@Right_LR:
 		bra	Object_BuildSpr
@Return:
 		rts
 	
; ----------------------------------------------
; Object_MarkGone
; ----------------------------------------------

Object_MarkGone:
; 		lea	(RAM_LevelBuffer),a5

;    		move.w	lvl_y(a5),d0
;    		move.w	d0,d2
;      		move.w	obj_y(a6),d1
;      		add.w	#$40,d1
;      		cmp.w	d0,d1
;      		blt.s	@its_gone
;        		add.w	#224+$40,d2
;       		move.w	obj_y(a6),d1
;       		cmp.w	d2,d1
;       		bgt.s	@its_gone
;      		
;   		move.w	lvl_x(a5),d0
;   		move.w	d0,d2
;     		move.w	obj_x(a6),d1
;     		add.w	#$40,d1
;     		cmp.w	d0,d1
;     		blt.s	@its_gone
;       		add.w	#320+$40,d2		;TODO: horizontal mode 
;      		move.w	obj_x(a6),d1
;      		cmp.w	d2,d1
;      		bgt.s	@its_gone
		rts

@its_gone:
		moveq	#0,d0
		move.b 	obj_spwnindx(a6),d0
		mulu.w	#$A,d0
		lea	(RAM_LevelObjPos),a5
		adda	d0,a5
		bclr	#7,(a5)			;Reset ON SCREEN flag
		clr.l	obj_code(a6)
		clr.b	obj_index(a6)
@Return:
 		rts
		
; ----------------------------------------------
; Object animation
; 
; Input
; d1 | LONG - Animation data
; 
; Output
; d0 | WORD - Frame
; 
; Uses:
; d2
; ----------------------------------------------
 
Object_Animate:
 		tst.l	d1
  		beq.s	@Return
 		moveq	#0,d2
 		move.b	obj_anim_id+1(a6),d2
 		cmp.b	obj_anim_id(a6),d2
 		beq.s	@SameThing
 		move.b	obj_anim_id(a6),obj_anim_id+1(a6)
 		clr.w	obj_anim_next(a6)
@SameThing:
 		move.b	obj_anim_id(a6),d2
 		cmp.b	#$FF,d2
 		beq.s	@Return
 		lsl.w	#1,d2
		movea.l	d1,a0
 		adda	(a0,d2.w),a0
 
 		move.b	(a0)+,d2
 		sub.b	#1,obj_anim_spd(a6)
 		bpl.s	@Return
		move.b	d2,obj_anim_spd(a6)
 
 		moveq	#0,d1
 		move.w	obj_anim_next(a6),d2
 		move.b	(a0),d1
 		adda	d2,a0
 
 		and.l	#$FFFF0000,d0
 		move.b	(a0),d0
 		cmp.b	#$FF,d0
 		beq.s	@NoAnim
 		cmp.b	#$FE,d0
 		beq.s	@GoToFrame
 		cmp.b	#$FD,d0
 		beq.s	@LastFrame
 		
 		move.b	d0,obj_frame(a6)
 		add.w	#1,obj_anim_next(a6)
@Return:
 		rts
 
@NoAnim:
 		move.w	#1,obj_anim_next(a6)
 		move.w	d1,d0
 		move.b	d0,obj_frame(a6)
		rts
@LastFrame:
 		clr.b	obj_anim_spd(a6)
		rts
@GoToFrame:
		clr.w	obj_anim_next(a6)
		move.b	1(a0),obj_anim_next+1(a6)
		rts
	
; ----------------------------------------------
; Object Collision
; ----------------------------------------------

; ************************
; Find floor collision
; CENTER
; 
; Input:
; (Nothing)
; 
; Output:
; d0 | LONG - Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a3-a4 | d3
; ************************

object_FindPrz_Floor:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
  		bra.s	objSearchCol_Floor
  		
object_FindCol_Floor:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4

objSearchCol_Floor:
		moveq	#0,d0
		moveq	#0,d3
 		move.w	obj_y(a6),d3
 		tst.w	d3
 		bmi	@no_col
 		move.w	lvl_y_size(a5),d4
 		lsl.w	#4,d4
 		cmp.w	d4,d3
 		bgt	@no_col
 		lsr.w	#4,d3
 		and.w	#$FFF,d3
 		move.w	d3,d0
 		lsl.l	#8,d0			;000YYY00
 		mulu.w	lvl_x_size(a5),d3
 		adda	d3,a4
 		
; ------------------------
; X check
; ------------------------

 		movea.l	a4,a3
 		move.w	obj_x(a6),d3
 		tst.w	d3
 		bmi.s	@force_on
 		move.w	lvl_x_size(a5),d4
 		lsl.w	#4,d4
   		cmp.w	d4,d3
   		bge.s	@no_col
   		
 		lsr.w	#4,d3
 		adda 	d3,a3
 		lsl.w	#4,d3
 		swap	d3
 		or.l	d3,d0			;XXXYYY00
 		or.b	(a3),d0	
 		rts
 		
@force_on:
 		moveq	#1,d0
@no_col:
		rts
		
; ************************
; Find side collision
; CENTER
;
; Input:
; a5 - RAM_LevelBuffer
; a3 - Collision data
; d3 | LONG - Object size (LEFT|RIGHT|UP|DOWN)
; d4 | LONG - X pos
; d5 | LONG - Y pos
; d6 | LONG - X Speed
; d7 | LONG - Y Speed
; 
; Output:
; d0 | LONG - TOP    Xpos|Ypos|ID $XXXYYYID
; d1 | LONG - BOTTOM Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a2-a3 | d3-d4
; ************************

object_FindPrz_Wall:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
  		bra.s	objSearchCol_Wall
  		
object_FindCol_Wall:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4

objSearchCol_Wall:
		moveq	#0,d0
		moveq	#0,d1
		moveq	#0,d3
 		move.w	obj_y(a6),d3
 		sub.w	#1,d3
 		tst.w	d3
 		bmi	@no_col
 		move.w	lvl_y_size(a5),d4
 		lsl.w	#4,d4
 		cmp.w	d4,d3
 		bge	@no_col
 		lsr.w	#4,d3
 		and.w	#$FFF,d3
 		move.w	d3,d0
 		lsl.l	#8,d0			;000YYY00
 		move.l	d0,d1
 		mulu.w	lvl_x_size(a5),d3
 		adda	d3,a4
 		
; ------------------------
; X check
; LEFT
; ------------------------

;  		bra.s	@No_col_side_l
		
  		movea.l	a4,a3

 		move.w	obj_x(a6),d3
 		tst.w	d3
 		bmi.s	@force_on
 		move.w	lvl_x_size(a5),d4
 		lsl.w	#4,d4
  		cmp.w	d4,d3
  		bge.s	@force_on
 		lsr.w	#4,d3
 		adda 	d3,a3
 		lsl.w	#4,d3
 		swap	d3
 		or.l	d3,d0			;XXXYYY00
 		
  		moveq	#0,d5
		move.w	d3,d5
		and.w	#$FFF0,d5
		swap	d5
		or.l	d5,d0
		
  		lsr.w	#4,d5
  		adda 	d5,a3
  		
		moveq	#0,d5
  		move.b	obj_size+2(a6),d5
  		lsr.w	#1,d5
  		sub.w	#1,d5
@Next_col_l:
		tst.b	(a3)
		beq.s	@nothing_l
;  		tst.w	d3
;  		bmi.s	@nothing_l
; 		move.w	d3,d4
; 		and.l	#$FFF0,d4
; 		lsl.l	#4,d4
; 		or.l	d4,d0
		move.b	(a3),d0

		moveq	#0,d5
@nothing_l:
		suba	lvl_x_size(a5),a3
		sub.w	#$10,d3
  		dbf	d5,@Next_col_l

@No_col_side_l:

; ------------------------
; X check
; ------------------------

 		movea.l	a4,a3
 		move.w	obj_x(a6),d3
 		tst.w	d3
 		bmi.s	@force_on
 		move.w	lvl_x_size(a5),d4
 		lsl.w	#4,d4
  		cmp.w	d4,d3
  		bge.s	@force_on
 		lsr.w	#4,d3
 		adda 	d3,a3
 		lsl.w	#4,d3
 		swap	d3
 		or.l	d3,d0			;XXXYYY00
 		move.b	(a3),d0	
 		rts
@force_on:
		moveq	#1,d0
@no_col:
		rts
		

; ************************
; Find Ceiling collision
; CENTER
; 
; Input:
; (Nothing)
; 
; Output:
; d0 | LONG - Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a3-a4 | d3-d4
; ************************

object_FindPrz_Ceiling:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
  		bra.s	objSearchCol_Ceiling
  		
object_FindCol_Ceiling:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4
  		
objSearchCol_Ceiling:
		moveq	#0,d0
		moveq	#0,d3
		moveq	#0,d4
 		move.w	obj_y(a6),d3
 		sub.w	#1,d3
    		move.b	obj_size+2(a6),d4
    		lsl.w	#3,d4
    		sub.w	d4,d3			;UP SIZE
    		sub.w	#1,d3
 		tst.w	d3
 		bmi	@no_col
 		move.w	lvl_y_size(a5),d4
 		lsl.w	#4,d4
 		cmp.w	d4,d3
 		bgt	@no_col
 		lsr.w	#4,d3
 		and.w	#$FFF,d3
 		move.w	d3,d0
 		lsl.l	#8,d0			;000YYY00
 		mulu.w	lvl_x_size(a5),d3
 		adda	d3,a4
		
; ------------------------
; X check
; ------------------------

 		movea.l	a4,a3
 		move.w	obj_x(a6),d3
 		sub.w	#1,d3
 		tst.w	d3
 		bmi.s	@no_col
 		move.w	lvl_x_size(a5),d4
 		lsl.w	#4,d4
   		cmp.w	d4,d3
   		bge.s	@no_col
 		lsr.w	#4,d3
 		adda 	d3,a3
 		lsl.w	#4,d3
 		swap	d3
 		or.l	d3,d0			;XXXYYY00
 		move.b	(a3),d0	
@no_col:
		rts
		
; ************************
; Find floor collision
; LEFT/RIGHT
;
; Input:
; (Nothing)
; 
; Output:
; d0 | LONG - RIGHT FEET Xpos|Ypos|ID $XXXYYYID
; d1 | LONG - LEFT FEET Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a3-a4 | d3-d4
; ************************

object_FindPrz_FloorSides:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
		bra.s	objSearchCol_FloorSides
		
object_FindCol_FloorSides:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4
  		
objSearchCol_FloorSides:
		moveq	#0,d0
		moveq	#0,d1
		moveq	#0,d3
 		move.w	obj_y(a6),d3
 		tst.w	d3
 		bmi	@no_col
  		move.w	lvl_y_size(a5),d4
  		lsl.w	#4,d4
  		cmp.w	d4,d3
  		bgt	@no_col
 		lsr.w	#4,d3
  		move.w	d3,d0
  		move.w	d3,d1
  		lsl.l	#8,d0			; LEFT 000YYY00
  		lsl.l	#8,d1			;RIGHT 000YYY00
 		mulu.w	lvl_x_size(a5),d3
 		adda	d3,a4

; ------------------------
; X check
; 
; LEFT
; ------------------------

 		moveq	#0,d3
		moveq	#0,d4
  		move.w	obj_x(a6),d4
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
      		cmp.w	d3,d4
      		blt.s	@dontfix_l
      		sub.w	#1,d4
@dontfix_l:
 		tst.w	d4
 		bmi.s	@zero_l
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		
 		swap	d4		; d4 - XRead | Free
 		move.b	obj_size(a6),d4
 		and.w	#$FF,d4		; d4 - XRead | Loop
 		tst.w	d4
 		beq.s	@zero_l
@Next_col_l:
 		tst.b	(a3)
 		beq.s	@No_col_l
 		
 		swap	d4
 		move.w	d4,d3
 		swap	d4
 		and.w	#$FFF0,d3
 		swap	d3
 		or.l	d3,d1		; LEFT XXXYYY00
		move.b	(a3),d1
 		bra.s	@zero_l
@No_col_l:
 		swap	d4		; d4 - Loop | Xread
 		sub.w	#8,d4
   		bpl.s	@Fine_l
   		clr.w	d4
@Fine_l:
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		swap	d4		; d4 - XRead | Loop
 		dbf	d4,@Next_col_l
@zero_l:
		
; ------------------------
; X check
; 
; RIGHT
; ------------------------

 		moveq	#0,d3
 		moveq	#0,d4
 		move.w	obj_x(a6),d4
 		sub.w	#1,d4
 		tst.w	d4
 		bmi.s	@no_col
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
   		cmp.w	d3,d4
    		bge.s	@no_col

 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		
 		swap	d4		;d4 - XRead | Free
 		move.b	obj_size+1(a6),d4
 		and.w	#$FF,d4		;d4 - XRead | Loop
 		tst.w	d4
 		beq.s	@no_col
 		
@Next_col_r:
 		swap	d4		;d4 - Loop | Xread
 		tst.b	(a3)
 		beq.s	@No_col_r
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
   		cmp.w	d3,d4
    		bge.s	@no_col
    		
 		move.w	d4,d3
 		and.w	#$FFF0,d3
 		swap	d3
 		or.l	d3,d0		; RIGHT XXXYYY00
		move.b	(a3),d0
 		rts
 		
@No_col_r:
 		add.w	#8,d4
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		swap	d4		;d4 - XRead | Loop
 		dbf	d4,@Next_col_r
 		
; ------------------------

@no_col:
		rts

; ************************
; Find side collision
; LEFT/RIGHT
;
; Input:
; (Nothing)
; 
; Output:
; d0 | LONG - RIGHT FEET Xpos|Ypos|ID $XXXYYYID
; d1 | LONG - LEFT FEET Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a3-a4 | d3-d4
; ************************

object_FindPrz_WallSides:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
  		bra.s	objSearchCol_WallSides
  		
object_FindCol_WallSides:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4
  		
objSearchCol_WallSides:
		moveq	#0,d0
		moveq	#0,d1
		moveq	#0,d3
  		moveq	#0,d4
 		move.w	obj_y(a6),d3
 		sub.w	#5,d3			;TODO: esto.
 		tst.w	d3
 		bmi	@no_col
 		move.w	lvl_y_size(a5),d4
 		lsl.w	#4,d4
 		cmp.w	d4,d3
 		bgt	@no_col
 		lsr.w	#4,d3
 		mulu.w	lvl_x_size(a5),d3
 		adda	d3,a4
		
; ------------------------
; X check
; LEFT
; ------------------------
	
  		moveq	#0,d3
  		movea.l	a4,a3
  		move.w	obj_x(a6),d4
   		sub.w	#1,d4
  		tst.w	d4
  		bmi.s	@No_col_side_l
  		move.b	obj_size(a6),d3
		tst.b	d3
		beq.s	@No_col_side_l
		lsl.b	#3,d3			; *8
		sub.w	#1,d3			; -1
		sub.w	d3,d4			; LEFT SIZE
  		move.b	#1,d1
  		tst.w	d4
  		bmi.s	@No_col_side_l 
 		move.w	lvl_x_size(a5),d3
  		lsl.w	#4,d3
   		cmp.w	d3,d4
   		bge.s	@No_col_side_l
   		clr.b	d1
   		
   		moveq	#0,d3
  		move.w	d4,d3
  		and.w	#$FFF0,d3
  		move.w	d3,d1
  		swap	d1
   		lsr.w	#4,d3
   		adda 	d3,a3
   		
   		move.w	lvl_x_size(a5),d3
   		swap	d3
 		move.w	obj_y(a6),d3		; d3 - Ypos
 		sub.w	#5,d3
		moveq	#0,d4
  		move.b	obj_size+2(a6),d4
  		and.w	#$FF,d4
  		lsr.w	#1,d4
   		tst.w	d4
   		beq.s	@Next_col_l
   		sub.w	#1,d4
@Next_col_l:
 		tst.w	d3
 		bmi.s	@No_col_side_l
		tst.b	(a3)
		beq.s	@nothing_l
		move.w	d3,d4
		and.l	#$FFF0,d4
		lsl.l	#4,d4
		or.l	d4,d1
		move.b	(a3),d1
		bra.s	@No_col_side_l
@nothing_l:
		swap	d3
		suba	d3,a3
		swap	d3
		sub.w	#$10,d3
  		dbf	d4,@Next_col_l

@No_col_side_l:
		
; ------------------------
; X check
; RIGHT
; ------------------------

  		moveq	#0,d3
  		movea.l	a4,a3
  		move.w	obj_x(a6),d4
    		add.w	#1,d4
  		tst.w	d4
  		bmi.s	@no_col
  		move.b	obj_size+1(a6),d3
		tst.w	d3
		beq.s	@no_col
		lsl.b	#3,d3			; *8
		sub.w	#1,d3
		add.w	d3,d4			; RIGHT SIZE
  		tst.w	d4
  		bmi.s	@no_col
   		move.b	#1,d0
  		move.w	lvl_x_size(a5),d3
  		lsl.w	#4,d3
   		cmp.w	d3,d4
   		bge.s	@no_col
    		clr.b	d0
    		
   		moveq	#0,d3
 		move.w	d4,d3
 		and.w	#$FFF0,d3
 		move.w	d3,d0
 		swap	d0
   		lsr.w	#4,d3
   		adda 	d3,a3
  		
  		moveq	#0,d3
		moveq	#0,d4
		move.w	obj_y(a6),d3		; 0000 YPOS
		sub.w	#5,d3
  		move.b	obj_size+2(a6),d4
  		lsr.w	#1,d4
   		tst.w	d4
   		beq.s	@Next_col_r
   		sub.w	#1,d4
@Next_col_r:
		tst.b	(a3)
		beq.s	@nothing_r
 		tst.w	d3
 		bmi.s	@nothing_r
		move.w	d3,d4
		and.l	#$FFF0,d4
		lsl.l	#4,d4
		or.l	d4,d0
		move.b	(a3),d0
		rts
@nothing_r:
		suba	lvl_x_size(a5),a3
		sub.w	#$10,d3
		dbf	d4,@Next_col_r
		
; ------------------------
; Finish checking
; ------------------------

@no_col:
		rts
		
; ************************
; Find ceiling collision
; LEFT/RIGHT
; 
; Input:
; (Nothing)
; 
; Output:
; d0 | LONG - RIGHT FEET Xpos|Ypos|ID $XXXYYYID
; d1 | LONG - LEFT FEET Xpos|Ypos|ID $XXXYYYID
; 
; Uses:
; a3-a4 | d3-d4
; ************************

object_FindPrz_CeilingSides:
		lea	(RAM_LevelBuffer),a5
  		lea	(RAM_LevelPrizes),a4
  		bra.s	objSearchCol_CeilingSides
  		
object_FindCol_CeilingSides:
		lea	(RAM_LevelBuffer),a5
  		movea.l	lvl_collision(a5),a4
  		
objSearchCol_CeilingSides:
		moveq	#0,d0
		moveq	#0,d1
 		moveq	#0,d3
 		clr.w	d0
  		move.w	obj_y(a6),d4
  		sub.w	#1,d4
   		move.b	obj_size+2(a6),d3
   		lsl.w	#3,d3
    		sub.w	#1,d3
   		sub.w	d3,d4			;UP SIZE
  		tst.w	d4
  		bmi	@no_col
 		move.w	lvl_y_size(a5),d3
 		lsl.w	#4,d3
 		cmp.w	d3,d4
 		bgt	@no_col
 		
  		lsr.w	#4,d4
  		move.w	d4,d0
  		move.w	d4,d1
  		lsl.l	#8,d0			; LEFT 000YYY00
  		lsl.l	#8,d1			;RIGHT 000YYY00
 		mulu.w	lvl_x_size(a5),d4
  		adda	d4,a4

; ------------------------
; X check
; 
; LEFT
; ------------------------
   		
		moveq	#0,d3
		moveq	#0,d4
		
 		move.w	obj_x(a6),d4
; 		moveq	#0,d3
;  		move.b	obj_size(a6),d3
;  		lsl.w	#3,d3
;  		sub.w	d3,d4
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
       		cmp.w	d3,d4
       		blt.s	@dontfix_l
       		sub.w	#1,d4
@dontfix_l:
 		tst.w	d4
 		bmi.s	@zero_l
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		
 		swap	d4		; d4 - XRead | Free
 		clr.w	d4
 		move.b	obj_size(a6),d4
 		and.w	#$FF,d4		; d4 - XRead | Loop
 		tst.w	d4
 		beq.s	@zero_l
@Next_col_l:
 		tst.b	(a3)
 		beq.s	@No_col_l
 		
 		swap	d4
 		move.w	d4,d3
 		swap	d4
 		and.w	#$FFF0,d3
 		swap	d3
 		or.l	d3,d1		; LEFT XXXYYY00
		move.b	(a3),d1
 		bra.s	@zero_l
@No_col_l:
 		swap	d4		; d4 - Loop | Xread
 		sub.w	#8,d4
   		bpl.s	@Fine_l
   		clr.w	d4
@Fine_l:
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		swap	d4		; d4 - XRead | Loop
 		dbf	d4,@Next_col_l
@zero_l:
		
; ------------------------
; X check
; 
; RIGHT
; ------------------------

 		moveq	#0,d3
 		moveq	#0,d4
 		move.w	obj_x(a6),d4
 		sub.w	#1,d4
 		tst.w	d4
 		bmi.s	@no_col
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
   		cmp.w	d3,d4
    		bge.s	@no_col
 		move.w	d4,d3
 		lsr.w	#4,d3
 		movea.l	a4,a3
 		adda 	d3,a3
 		
 		swap	d4		;d4 - XRead | Free
 		move.b	obj_size+1(a6),d4
 		and.w	#$FF,d4		;d4 - XRead | Loop
 		tst.w	d4
 		beq.s	@no_col
 		
@Next_col_r:
 		swap	d4		;d4 - Loop | Xread
 		tst.b	(a3)
 		beq.s	@No_col_r
 		move.w	lvl_x_size(a5),d3
 		lsl.w	#4,d3
   		cmp.w	d3,d4
    		bge.s	@no_col
    		
 		move.w	d4,d3
 		and.w	#$FFF0,d3
 		swap	d3
 		or.l	d3,d0		; RIGHT XXXYYY00
		move.b	(a3),d0
    		rts
    		
@No_col_r:
 		movea.l	a4,a3

 		add.w	#8,d4
 		move.w	d4,d3
 		lsr.w	#4,d3
 		adda 	d3,a3
 		swap	d4		;d4 - XRead | Loop
 		dbf	d4,@Next_col_r
 		
; ------------------------

@no_col:
		rts
 	
 	
; ************************
; Sets a object in the floor
; DOWN/UP
; CENTER
; 
; Input:
; d0 - Floor ID
; (Object's current Ypos)
; 
; Output:
; d7 | LONG - Y speed result
; 
; Uses:
; a3-a4 | d3-d4
; ************************

object_SetCol_Floor:
		cmp.b	#1,d0
		beq	@floorsolid

   		tst.l	d7
   		bmi.s	@NoCol
   		
   		move.l	obj_y(a6),d1
   		move.l	d1,d3
  		lea	(col_SlopeData),a3
  		and.w	#$FF,d0
  		move.w	d0,d1
  		lsl.w	#4,d1
  		adda	d1,a3
 		move.l	obj_x(a6),d0
 		swap	d0
 		and.w	#$F,d0
 		move.b	(a3,d0.w),d0
    		and.w	#$F,d0
    		and.l	#$FFF00000,d1
  		swap	d0
  		and.l	#$FFFF0000,d0
 		add.l	d0,d1
 		
		btst	#bitobj_air,obj_status(a6)
		beq.s	@dontchk
  		add.l	d7,d3
  		cmp.l	d1,d3
  		blt.s	@NoCol
@dontchk:
  		move.l	d1,obj_y(a6)

   		bclr	#bitobj_air,obj_status(a6)
 		clr.l	d7
     		tst.l	d6
     		beq.s	@NoCol
    		move.l	#$10000,d7
@NoCol:
		rts

@floorsolid:
		bclr	#bitobj_air,obj_status(a6)
		and.l	#$FFF00000,obj_y(a6)
		clr.l	d7
		rts

; =================================================================
; ------------------------------------------------
; Includes
; ------------------------------------------------

 		include	"engine/modes/level/data/objects/player/code.asm"
  		include	"engine/modes/level/data/objects/chamoy/code.asm"
